@@ -8,12 +8,11 @@ pub(crate) mod command;
 pub(crate) mod commands;
 pub mod response;
 
-use anyhow::Result;
 use std::convert::TryFrom;
 
 use crate::apdu::command::Expect;
 use crate::apdu::{command::Command, response::RawResponse};
-use crate::{CardClient, Error, StatusBytes};
+use crate::{CardTransaction, Error, StatusBytes};
 
 /// "Maximum amount of bytes in a short APDU command or response" (from pcsc)
 const MAX_BUFFER_SIZE: usize = 264;
@@ -22,13 +21,16 @@ const MAX_BUFFER_SIZE: usize = 264;
 ///
 /// If the reply is truncated, this fn assembles all the parts and returns
 /// them as one aggregated Response.
-pub(crate) fn send_command(
-    card_client: &mut dyn CardClient,
+pub(crate) fn send_command<C>(
+    card_tx: &mut C,
     cmd: Command,
     expect_reply: bool,
-) -> Result<RawResponse, Error> {
+) -> Result<RawResponse, Error>
+where
+    C: CardTransaction + ?Sized,
+{
     let mut resp = RawResponse::try_from(send_command_low_level(
-        card_client,
+        card_tx,
         cmd.clone(),
         if expect_reply {
             Expect::Some
@@ -38,11 +40,7 @@ pub(crate) fn send_command(
     )?)?;
 
     if let StatusBytes::UnknownStatus(0x6c, size) = resp.status() {
-        resp = RawResponse::try_from(send_command_low_level(
-            card_client,
-            cmd,
-            Expect::Short(size),
-        )?)?;
+        resp = RawResponse::try_from(send_command_low_level(card_tx, cmd, Expect::Short(size))?)?;
     }
 
     while let StatusBytes::OkBytesAvailable(bytes) = resp.status() {
@@ -51,17 +49,14 @@ pub(crate) fn send_command(
 
         // Get next chunk of data
         let next = RawResponse::try_from(send_command_low_level(
-            card_client,
+            card_tx,
             commands::get_response(),
             Expect::Short(bytes),
         )?)?;
 
         match next.status() {
             StatusBytes::OkBytesAvailable(_) | StatusBytes::Ok => {
-                log::debug!(
-                    " appending {} bytes to response",
-                    next.raw_data().len()
-                );
+                log::debug!(" appending {} bytes to response", next.raw_data().len());
 
                 // Append new data to resp.data and overwrite status.
                 resp.raw_mut_data().extend_from_slice(next.raw_data());
@@ -81,13 +76,16 @@ pub(crate) fn send_command(
 ///
 /// If the response is chained, this fn only returns one chunk, the caller
 /// needs to re-assemble the chained response-parts.
-fn send_command_low_level(
-    card_client: &mut dyn CardClient,
+fn send_command_low_level<C>(
+    card_tx: &mut C,
     cmd: Command,
     expect_response: Expect,
-) -> Result<Vec<u8>, Error> {
+) -> Result<Vec<u8>, Error>
+where
+    C: CardTransaction + ?Sized,
+{
     let (ext_support, chaining_support, mut max_cmd_bytes, max_rsp_bytes) =
-        if let Some(caps) = card_client.card_caps() {
+        if let Some(caps) = card_tx.card_caps() {
             log::debug!("found card caps data!");
 
             (
@@ -103,13 +101,13 @@ fn send_command_low_level(
             (false, false, 255, 255)
         };
 
-    // If the CardClient implementation has an inherent limit for the cmd
+    // If the CardTransaction implementation has an inherent limit for the cmd
     // size, take that limit into account.
-    // (E.g. when using scdaemon as a CardClient backend, there is a
+    // (E.g. when using scdaemon as a CardTransaction backend, there is a
     // limitation to 1000 bytes length for Assuan commands, which
     // translates to maximum command length of a bit under 500 bytes)
-    if let Some(max_cardclient_cmd_bytes) = card_client.max_cmd_len() {
-        max_cmd_bytes = usize::min(max_cmd_bytes, max_cardclient_cmd_bytes);
+    if let Some(max_card_cmd_bytes) = card_tx.max_cmd_len() {
+        max_cmd_bytes = usize::min(max_cmd_bytes, max_card_cmd_bytes);
     }
 
     log::debug!(
@@ -152,16 +150,13 @@ fn send_command_low_level(
             let last = i == chunks.len() - 1;
 
             let cla = if last { 0x00 } else { 0x10 };
-            let partial =
-                Command::new(cla, cmd.ins(), cmd.p1(), cmd.p2(), d.to_vec());
+            let partial = Command::new(cla, cmd.ins(), cmd.p1(), cmd.p2(), d.to_vec());
 
-            let serialized = partial
-                .serialize(ext_len, expect_response)
-                .map_err(Error::InternalError)?;
+            let serialized = partial.serialize(ext_len, expect_response)?;
 
             log::debug!(" -> chained APDU command: {:x?}", &serialized);
 
-            let resp = card_client.transmit(&serialized, buf_size)?;
+            let resp = card_tx.transmit(&serialized, buf_size)?;
 
             log::debug!(" <- APDU response: {:x?}", &resp);
 
@@ -178,8 +173,7 @@ fn send_command_low_level(
 
                 // ISO: "If SW1-SW2 is set to '6883', then the last
                 // command of the chain is expected."
-                if !(status == StatusBytes::Ok
-                    || status == StatusBytes::LastCommandOfChainExpected)
+                if !(status == StatusBytes::Ok || status == StatusBytes::LastCommandOfChainExpected)
                 {
                     // Unexpected status for a non-final chunked response
                     return Err(status.into());
@@ -204,7 +198,7 @@ fn send_command_low_level(
 
         log::debug!(" -> APDU command: {:x?}", &serialized);
 
-        let resp = card_client.transmit(&serialized, buf_size)?;
+        let resp = card_tx.transmit(&serialized, buf_size)?;
 
         log::debug!(" <- APDU response: {:x?}", resp);
 
