@@ -3,10 +3,9 @@
 
 //! OpenPGP card data objects (DO)
 
-use anyhow::{anyhow, Result};
 use chrono::{DateTime, Utc};
-use std::convert::TryFrom;
-use std::convert::TryInto;
+use std::convert::{TryFrom, TryInto};
+use std::fmt::{Display, Formatter};
 use std::time::{Duration, UNIX_EPOCH};
 
 use crate::{algorithm::Algo, tlv::Tlv, Error, KeySet, KeyType};
@@ -41,7 +40,7 @@ impl ApplicationRelatedData {
         if let Some(aid) = aid {
             Ok(ApplicationIdentifier::try_from(&aid.serialize()[..])?)
         } else {
-            Err(anyhow!("Couldn't get Application ID.").into())
+            Err(Error::NotFound("Couldn't get Application ID.".to_string()))
         }
     }
 
@@ -54,15 +53,15 @@ impl ApplicationRelatedData {
             log::debug!("Historical bytes: {:x?}", hist);
             (hist.serialize().as_slice()).try_into()
         } else {
-            Err(anyhow!("Failed to get historical bytes.").into())
+            Err(Error::NotFound(
+                "Failed to get historical bytes.".to_string(),
+            ))
         }
     }
 
     /// Get extended length information (ISO 7816-4), which
     /// contains maximum number of bytes for command and response.
-    pub fn extended_length_information(
-        &self,
-    ) -> Result<Option<ExtendedLengthInfo>> {
+    pub fn extended_length_information(&self) -> Result<Option<ExtendedLengthInfo>, Error> {
         // get from cached "application related data"
         let eli = self.0.find(&[0x7f, 0x66].into());
 
@@ -88,9 +87,7 @@ impl ApplicationRelatedData {
     }
 
     /// Get extended Capabilities
-    pub fn extended_capabilities(
-        &self,
-    ) -> Result<ExtendedCapabilities, Error> {
+    pub fn extended_capabilities(&self) -> Result<ExtendedCapabilities, Error> {
         // FIXME: caching?
         let app_id = self.application_id()?;
         let version = app_id.version();
@@ -104,27 +101,29 @@ impl ApplicationRelatedData {
                 version,
             ))?)
         } else {
-            Err(anyhow!("Failed to get extended capabilities.").into())
+            Err(Error::NotFound(
+                "Failed to get extended capabilities.".to_string(),
+            ))
         }
     }
 
     /// Get algorithm attributes (for each key type)
-    pub fn algorithm_attributes(&self, key_type: KeyType) -> Result<Algo> {
+    pub fn algorithm_attributes(&self, key_type: KeyType) -> Result<Algo, Error> {
         // get from cached "application related data"
         let aa = self.0.find(&[key_type.algorithm_tag()].into());
 
         if let Some(aa) = aa {
             Algo::try_from(&aa.serialize()[..])
         } else {
-            Err(anyhow!(
+            Err(Error::NotFound(format!(
                 "Failed to get algorithm attributes for {:?}.",
                 key_type
-            ))
+            )))
         }
     }
 
     /// Get PW status Bytes
-    pub fn pw_status_bytes(&self) -> Result<PWStatusBytes> {
+    pub fn pw_status_bytes(&self) -> Result<PWStatusBytes, Error> {
         // get from cached "application related data"
         let psb = self.0.find(&[0xc4].into());
 
@@ -135,7 +134,9 @@ impl ApplicationRelatedData {
 
             Ok(pws)
         } else {
-            Err(anyhow!("Failed to get PW status Bytes."))
+            Err(Error::NotFound(
+                "Failed to get PW status Bytes.".to_string(),
+            ))
         }
     }
 
@@ -152,25 +153,24 @@ impl ApplicationRelatedData {
 
             Ok(fp)
         } else {
-            Err(anyhow!("Failed to get fingerprints.").into())
+            Err(Error::NotFound("Failed to get fingerprints.".into()))
         }
     }
 
     /// Generation dates/times of key pairs
-    pub fn key_generation_times(
-        &self,
-    ) -> Result<KeySet<KeyGenerationTime>, Error> {
+    pub fn key_generation_times(&self) -> Result<KeySet<KeyGenerationTime>, crate::Error> {
         let kg = self.0.find(&[0xcd].into());
 
         if let Some(kg) = kg {
-            let kg: KeySet<KeyGenerationTime> =
-                (&kg.serialize()[..]).try_into()?;
+            let kg: KeySet<KeyGenerationTime> = (&kg.serialize()[..]).try_into()?;
 
             log::debug!("Key generation: {:x?}", kg);
 
             Ok(kg)
         } else {
-            Err(anyhow!("Failed to get key generation times.").into())
+            Err(Error::NotFound(format!(
+                "Failed to get key generation times."
+            )))
         }
     }
 }
@@ -269,9 +269,9 @@ pub struct ExtendedCapabilities {
     max_cmd_len: Option<u16>,  // v2
     max_resp_len: Option<u16>, // v2
 
-    max_len_special_do: Option<u16>, // v3
+    max_len_special_do: Option<u16>,          // v3
     pin_block_2_format_support: Option<bool>, // v3
-    mse_command_support: Option<bool>, // v3
+    mse_command_support: Option<bool>,        // v3
 }
 
 /// 4.1.3.1 Extended length information
@@ -284,8 +284,8 @@ pub struct ExtendedLengthInfo {
 /// Cardholder Related Data (see spec pg. 22)
 #[derive(Debug, PartialEq)]
 pub struct CardholderRelatedData {
-    name: Option<String>,
-    lang: Option<Vec<[char; 2]>>,
+    name: Option<Vec<u8>>,
+    lang: Option<Vec<Lang>>,
     sex: Option<Sex>,
 }
 
@@ -298,6 +298,58 @@ pub enum Sex {
     NotApplicable,
 }
 
+#[derive(Debug, PartialEq, Clone, Copy)]
+pub enum Lang {
+    Value([u8; 2]),
+    Invalid(u8),
+}
+
+impl Display for Lang {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        match self {
+            Self::Value(v) => {
+                write!(f, "{}{}", v[0] as char, v[1] as char)
+            }
+            Self::Invalid(v) => {
+                write!(f, "{:x?}", v)
+            }
+        }
+    }
+}
+
+impl From<(char, char)> for Lang {
+    fn from(c: (char, char)) -> Self {
+        Lang::Value([c.0 as u8, c.1 as u8])
+    }
+}
+
+impl From<[char; 2]> for Lang {
+    fn from(c: [char; 2]) -> Self {
+        Lang::Value([c[0] as u8, c[1] as u8])
+    }
+}
+
+impl From<Lang> for Vec<u8> {
+    fn from(lang: Lang) -> Self {
+        match lang {
+            Lang::Value(v) => vec![v[0], v[1]],
+            Lang::Invalid(v) => vec![v],
+        }
+    }
+}
+
+impl From<&[u8; 1]> for Lang {
+    fn from(data: &[u8; 1]) -> Self {
+        Lang::Invalid(data[0])
+    }
+}
+
+impl From<&[u8; 2]> for Lang {
+    fn from(data: &[u8; 2]) -> Self {
+        Lang::Value([data[0], data[1]])
+    }
+}
+
 impl From<&Sex> for u8 {
     fn from(sex: &Sex) -> u8 {
         match sex {
@@ -414,14 +466,14 @@ impl Fingerprint {
 }
 
 /// Helper fn for nom parsing
-pub(crate) fn complete<O>(
-    result: nom::IResult<&[u8], O>,
-) -> Result<O, anyhow::Error> {
-    let (rem, output) =
-        result.map_err(|err| anyhow!("Parsing failed: {:?}", err))?;
+pub(crate) fn complete<O>(result: nom::IResult<&[u8], O>) -> Result<O, Error> {
+    let (rem, output) = result.map_err(|_err| Error::ParseError(format!("Parsing failed")))?;
     if rem.is_empty() {
         Ok(output)
     } else {
-        Err(anyhow!("Parsing incomplete -- trailing data: {:x?}", rem))
+        Err(Error::ParseError(format!(
+            "Parsing incomplete, trailing data: {:x?}",
+            rem
+        )))
     }
 }
