@@ -5,7 +5,6 @@
 
 use std::convert::TryFrom;
 
-use anyhow::Result;
 use nom::branch::alt;
 use nom::bytes::complete::tag;
 use nom::combinator::map;
@@ -104,8 +103,7 @@ fn parse_ecdh(input: &[u8]) -> nom::IResult<&[u8], Algo> {
     let (input, _) = bytes::tag([0x12])(input)?;
     let (input, curve) = parse_oid(input)?;
 
-    let (input, import_format) =
-        alt((parse_import_format, default_import_format))(input)?;
+    let (input, import_format) = alt((parse_import_format, default_import_format))(input)?;
 
     Ok((
         input,
@@ -117,8 +115,7 @@ fn parse_ecdsa(input: &[u8]) -> nom::IResult<&[u8], Algo> {
     let (input, _) = bytes::tag([0x13])(input)?;
     let (input, curve) = parse_oid(input)?;
 
-    let (input, import_format) =
-        alt((parse_import_format, default_import_format))(input)?;
+    let (input, import_format) = alt((parse_import_format, default_import_format))(input)?;
 
     Ok((
         input,
@@ -130,8 +127,7 @@ fn parse_eddsa(input: &[u8]) -> nom::IResult<&[u8], Algo> {
     let (input, _) = bytes::tag([0x16])(input)?;
     let (input, curve) = parse_oid(input)?;
 
-    let (input, import_format) =
-        alt((parse_import_format, default_import_format))(input)?;
+    let (input, import_format) = alt((parse_import_format, default_import_format))(input)?;
 
     Ok((
         input,
@@ -144,9 +140,9 @@ pub(crate) fn parse(input: &[u8]) -> nom::IResult<&[u8], Algo> {
 }
 
 impl TryFrom<&[u8]> for Algo {
-    type Error = anyhow::Error;
+    type Error = crate::Error;
 
-    fn try_from(data: &[u8]) -> Result<Self> {
+    fn try_from(data: &[u8]) -> Result<Self, crate::Error> {
         complete(parse(data))
     }
 }
