@@ -4,20 +4,18 @@
 //! 4.1.3.1 Extended length information
 //! (Introduced in V3.0)
 
-use anyhow::Result;
 use nom::{bytes::complete::tag, number::complete as number, sequence};
 
 use crate::card_do::{complete, ExtendedLengthInfo};
 use std::convert::TryFrom;
 
 fn parse(input: &[u8]) -> nom::IResult<&[u8], (u16, u16)> {
-    let (input, (_, cmd, _, resp)) =
-        nom::combinator::all_consuming(sequence::tuple((
-            tag([0x2, 0x2]),
-            number::be_u16,
-            tag([0x2, 0x2]),
-            number::be_u16,
-        )))(input)?;
+    let (input, (_, cmd, _, resp)) = nom::combinator::all_consuming(sequence::tuple((
+        tag([0x2, 0x2]),
+        number::be_u16,
+        tag([0x2, 0x2]),
+        number::be_u16,
+    )))(input)?;
 
     Ok((input, (cmd, resp)))
 }
@@ -33,7 +31,7 @@ impl ExtendedLengthInfo {
 }
 
 impl TryFrom<&[u8]> for ExtendedLengthInfo {
-    type Error = anyhow::Error;
+    type Error = crate::Error;
 
     fn try_from(input: &[u8]) -> Result<Self, Self::Error> {
         let eli = complete(parse(input))?;
@@ -53,8 +51,8 @@ mod test {
     fn test_floss34() {
         let data = [0x2, 0x2, 0x8, 0x0, 0x2, 0x2, 0x8, 0x0];
 
-        let eli = ExtendedLengthInfo::try_from(&data[..])
-            .expect("failed to parse extended length info");
+        let eli =
+            ExtendedLengthInfo::try_from(&data[..]).expect("failed to parse extended length info");
 
         assert_eq!(
             eli,
