@@ -3,12 +3,10 @@
 
 //! Generation date/time of key pair (see spec pg. 24)
 
-use anyhow::anyhow;
-use chrono::{DateTime, NaiveDateTime, Utc};
-use nom::{combinator, number::complete as number, sequence};
-
 use crate::card_do::{KeyGenerationTime, KeySet};
 use crate::Error;
+use chrono::{DateTime, NaiveDateTime, Utc};
+use nom::{combinator, number::complete as number, sequence};
 use std::convert::TryFrom;
 
 impl From<KeyGenerationTime> for DateTime<Utc> {
@@ -35,18 +33,14 @@ fn gen_time(input: &[u8]) -> nom::IResult<&[u8], u32> {
     (number::be_u32)(input)
 }
 
-fn key_generation(
-    input: &[u8],
-) -> nom::IResult<&[u8], Option<KeyGenerationTime>> {
+fn key_generation(input: &[u8]) -> nom::IResult<&[u8], Option<KeyGenerationTime>> {
     combinator::map(gen_time, |kg| match kg {
         0 => None,
         kg => Some(KeyGenerationTime(kg)),
     })(input)
 }
 
-fn key_generation_set(
-    input: &[u8],
-) -> nom::IResult<&[u8], KeySet<KeyGenerationTime>> {
+fn key_generation_set(input: &[u8]) -> nom::IResult<&[u8], KeySet<KeyGenerationTime>> {
     combinator::into(sequence::tuple((
         key_generation,
         key_generation,
@@ -73,8 +67,7 @@ impl TryFrom<&[u8]> for KeySet<KeyGenerationTime> {
         // hasn't been completely consumed.
         self::key_generation_set(input)
             .map(|res| res.1)
-            .map_err(|err| anyhow!("Parsing failed: {:?}", err))
-            .map_err(Error::InternalError)
+            .map_err(|_err| Error::ParseError(format!("Parsing failed")))
     }
 }
 
@@ -86,8 +79,7 @@ mod test {
     #[test]
     fn test() {
         let data3 = [
-            0x60, 0xf3, 0xff, 0x71, 0x60, 0xf3, 0xff, 0x72, 0x60, 0xf3, 0xff,
-            0x83,
+            0x60, 0xf3, 0xff, 0x71, 0x60, 0xf3, 0xff, 0x72, 0x60, 0xf3, 0xff, 0x83,
         ];
 
         let fp_set: KeySet<KeyGenerationTime> = (&data3[..])
