@@ -3,22 +3,20 @@
 
 //! Generate and import keys
 
-use anyhow::{anyhow, Result};
 use std::convert::TryFrom;
 use std::time::{SystemTime, UNIX_EPOCH};
 
 use crate::algorithm::{Algo, AlgoInfo, Curve, EccAttrs, RsaAttrs};
 use crate::apdu::command::Command;
 use crate::apdu::commands;
-use crate::card_app::CardApp;
 use crate::card_do::{ApplicationRelatedData, Fingerprint, KeyGenerationTime};
 use crate::crypto_data::{
-    CardUploadableKey, EccKey, EccPub, EccType, PrivateKeyMaterial,
-    PublicKeyMaterial, RSAKey, RSAPub,
+    CardUploadableKey, EccKey, EccPub, EccType, PrivateKeyMaterial, PublicKeyMaterial, RSAKey,
+    RSAPub,
 };
+use crate::openpgp::OpenPgpTransaction;
 use crate::tlv::{length::tlv_encode_length, value::Value, Tlv};
-use crate::Error;
-use crate::{apdu, KeyType};
+use crate::{apdu, Error, KeyType};
 
 /// Generate asymmetric key pair on the card.
 ///
@@ -31,24 +29,20 @@ use crate::{apdu, KeyType};
 /// `fp_from_pub` calculates the fingerprint for a public key data object and
 /// creation timestamp
 pub(crate) fn gen_key_with_metadata(
-    card_app: &mut CardApp,
-    fp_from_pub: fn(
-        &PublicKeyMaterial,
-        KeyGenerationTime,
-        KeyType,
-    ) -> Result<Fingerprint, Error>,
+    card_tx: &mut OpenPgpTransaction,
+    fp_from_pub: fn(&PublicKeyMaterial, KeyGenerationTime, KeyType) -> Result<Fingerprint, Error>,
     key_type: KeyType,
     algo: Option<&Algo>,
 ) -> Result<(PublicKeyMaterial, KeyGenerationTime), Error> {
     // Set algo on card if it's Some
     if let Some(target_algo) = algo {
         // FIXME: caching
-        let ard = card_app.application_related_data()?; // no caching, here!
+        let ard = card_tx.application_related_data()?; // no caching, here!
         let ecap = ard.extended_capabilities()?;
 
         // Only set algo if card supports setting of algo attr
         if ecap.algo_attrs_changeable() {
-            card_app.set_algorithm_attributes(key_type, target_algo)?;
+            card_tx.set_algorithm_attributes(key_type, target_algo)?;
         } else {
             // Check if the current algo on the card is the one we want, if
             // not we return an error.
@@ -65,11 +59,11 @@ pub(crate) fn gen_key_with_metadata(
     }
 
     // get current (possibly updated) state of algo
-    let ard = card_app.application_related_data()?; // no caching, here!
+    let ard = card_tx.application_related_data()?; // no caching, here!
     let cur_algo = ard.algorithm_attributes(key_type)?;
 
     // generate key
-    let tlv = generate_asymmetric_key_pair(card_app, key_type)?;
+    let tlv = generate_asymmetric_key_pair(card_tx, key_type)?;
 
     // derive pubkey
     let pubkey = tlv_to_pubkey(&tlv, &cur_algo)?;
@@ -82,22 +76,22 @@ pub(crate) fn gen_key_with_metadata(
     // Store creation timestamp (unix time format, limited to u32)
     let ts = time
         .duration_since(UNIX_EPOCH)
-        .map_err(|e| Error::InternalError(anyhow!(e)))?
+        .map_err(|e| Error::InternalError(format!("This should never happen {}", e)))?
         .as_secs() as u32;
 
     let ts = ts.into();
 
-    card_app.set_creation_time(ts, key_type)?;
+    card_tx.set_creation_time(ts, key_type)?;
 
     // calculate/store fingerprint
     let fp = fp_from_pub(&pubkey, ts, key_type)?;
-    card_app.set_fingerprint(fp, key_type)?;
+    card_tx.set_fingerprint(fp, key_type)?;
 
     Ok((pubkey, ts))
 }
 
 /// Transform a public key Tlv from the card into PublicKeyMaterial
-fn tlv_to_pubkey(tlv: &Tlv, algo: &Algo) -> Result<PublicKeyMaterial> {
+fn tlv_to_pubkey(tlv: &Tlv, algo: &Algo) -> Result<PublicKeyMaterial, crate::Error> {
     let n = tlv.find(&[0x81].into());
     let v = tlv.find(&[0x82].into());
 
@@ -116,10 +110,10 @@ fn tlv_to_pubkey(tlv: &Tlv, algo: &Algo) -> Result<PublicKeyMaterial> {
             Ok(PublicKeyMaterial::E(ecc))
         }
 
-        (_, _, _) => Err(anyhow!(
+        (_, _, _) => Err(Error::UnsupportedAlgo(format!(
             "Unexpected public key material from card {:?}",
             tlv
-        )),
+        ))),
     }
 }
 
@@ -128,14 +122,14 @@ fn tlv_to_pubkey(tlv: &Tlv, algo: &Algo) -> Result<PublicKeyMaterial> {
 /// This runs the low level key generation primitive on the card.
 /// (This does not set algorithm attributes, creation time or fingerprint)
 pub(crate) fn generate_asymmetric_key_pair(
-    card_app: &mut CardApp,
+    card_tx: &mut OpenPgpTransaction,
     key_type: KeyType,
 ) -> Result<Tlv, Error> {
     // generate key
     let crt = control_reference_template(key_type)?;
     let gen_key_cmd = commands::gen_key(crt.serialize().to_vec());
 
-    let resp = apdu::send_command(card_app.card_client(), gen_key_cmd, true)?;
+    let resp = apdu::send_command(card_tx.tx(), gen_key_cmd, true)?;
     resp.check_ok()?;
 
     let tlv = Tlv::try_from(resp.data()?)?;
@@ -150,19 +144,18 @@ pub(crate) fn generate_asymmetric_key_pair(
 ///
 /// (See 7.2.14 GENERATE ASYMMETRIC KEY PAIR)
 pub(crate) fn public_key(
-    card_app: &mut CardApp,
+    card_tx: &mut OpenPgpTransaction,
     key_type: KeyType,
 ) -> Result<PublicKeyMaterial, Error> {
     // get current algo
-    let ard = card_app.application_related_data()?; // FIXME: caching
+    let ard = card_tx.application_related_data()?; // FIXME: caching
     let algo = ard.algorithm_attributes(key_type)?;
 
     // get public key
     let crt = control_reference_template(key_type)?;
     let get_pub_key_cmd = commands::get_pub_key(crt.serialize().to_vec());
 
-    let resp =
-        apdu::send_command(card_app.card_client(), get_pub_key_cmd, true)?;
+    let resp = apdu::send_command(card_tx.tx(), get_pub_key_cmd, true)?;
     resp.check_ok()?;
 
     let tlv = Tlv::try_from(resp.data()?)?;
@@ -177,13 +170,13 @@ pub(crate) fn public_key(
 /// caused by checks before attempting to upload the key to the card, or by
 /// an error that the card reports during an attempt to upload the key).
 pub(crate) fn key_import(
-    card_app: &mut CardApp,
+    card_tx: &mut OpenPgpTransaction,
     key: Box<dyn CardUploadableKey>,
     key_type: KeyType,
     algo_info: Option<AlgoInfo>,
 ) -> Result<(), Error> {
     // FIXME: caching?
-    let ard = card_app.application_related_data()?;
+    let ard = card_tx.application_related_data()?;
 
     let (algo, key_cmd) = match key.private_key()? {
         PrivateKeyMaterial::R(rsa_key) => {
@@ -191,20 +184,15 @@ pub(crate) fn key_import(
             // (round up to 4-bytes, in case the key has 8+ leading zero bits)
             let rsa_bits = (((rsa_key.n().len() * 8 + 31) / 32) * 32) as u16;
 
-            let rsa_attrs =
-                determine_rsa_attrs(rsa_bits, key_type, &ard, algo_info)?;
+            let rsa_attrs = determine_rsa_attrs(rsa_bits, key_type, &ard, algo_info)?;
 
             let key_cmd = rsa_key_import_cmd(key_type, rsa_key, &rsa_attrs)?;
 
             (Algo::Rsa(rsa_attrs), key_cmd)
         }
         PrivateKeyMaterial::E(ecc_key) => {
-            let ecc_attrs = determine_ecc_attrs(
-                ecc_key.oid(),
-                ecc_key.ecc_type(),
-                key_type,
-                algo_info,
-            )?;
+            let ecc_attrs =
+                determine_ecc_attrs(ecc_key.oid(), ecc_key.ecc_type(), key_type, algo_info)?;
 
             let key_cmd = ecc_key_import_cmd(key_type, ecc_key, &ecc_attrs)?;
 
@@ -219,12 +207,12 @@ pub(crate) fn key_import(
 
     // Only set algo attrs if "Extended Capabilities" lists the feature
     if ard.extended_capabilities()?.algo_attrs_changeable() {
-        card_app.set_algorithm_attributes(key_type, &algo)?;
+        card_tx.set_algorithm_attributes(key_type, &algo)?;
     }
 
-    apdu::send_command(card_app.card_client(), key_cmd, false)?.check_ok()?;
-    card_app.set_fingerprint(fp, key_type)?;
-    card_app.set_creation_time(key.timestamp(), key_type)?;
+    apdu::send_command(card_tx.tx(), key_cmd, false)?.check_ok()?;
+    card_tx.set_fingerprint(fp, key_type)?;
+    card_tx.set_creation_time(key.timestamp(), key_type)?;
 
     Ok(())
 }
@@ -240,7 +228,7 @@ pub(crate) fn determine_rsa_attrs(
     key_type: KeyType,
     ard: &ApplicationRelatedData,
     algo_info: Option<AlgoInfo>,
-) -> Result<RsaAttrs> {
+) -> Result<RsaAttrs, crate::Error> {
     // Figure out suitable RSA algorithm parameters:
 
     // Does the card offer a list of algorithms?
@@ -284,16 +272,16 @@ pub(crate) fn determine_ecc_attrs(
     ecc_type: EccType,
     key_type: KeyType,
     algo_info: Option<AlgoInfo>,
-) -> Result<EccAttrs> {
+) -> Result<EccAttrs, crate::Error> {
     // If we have an algo_info, refuse upload if oid is not listed
     if let Some(algo_info) = algo_info {
         let algos = check_card_algo_ecc(algo_info, key_type, oid);
         if algos.is_empty() {
             // If oid is not in algo_info, return error.
-            return Err(anyhow!(
+            return Err(Error::UnsupportedAlgo(format!(
                 "Oid {:?} unsupported according to algo_info",
                 oid
-            ));
+            )));
         }
 
         // Note: Looking up ecc_type in the card's "Algorithm Information"
@@ -320,11 +308,7 @@ pub(crate) fn determine_ecc_attrs(
 }
 
 /// Look up RsaAttrs parameters in algo_info based on key_type and rsa_bits
-fn card_algo_rsa(
-    algo_info: AlgoInfo,
-    key_type: KeyType,
-    rsa_bits: u16,
-) -> Result<RsaAttrs, Error> {
+fn card_algo_rsa(algo_info: AlgoInfo, key_type: KeyType, rsa_bits: u16) -> Result<RsaAttrs, Error> {
     // Find suitable algorithm parameters (from card's list of algorithms).
 
     // Get Algos for this keytype
@@ -350,20 +334,14 @@ fn card_algo_rsa(
         Ok((**algo.last().unwrap()).clone())
     } else {
         // RSA with this bit length is not in algo_info
-        return Err(anyhow!(
-            "RSA {} unsupported according to algo_info",
-            rsa_bits
-        )
-        .into());
+        return Err(Error::UnsupportedAlgo(
+            format!("RSA {} unsupported according to algo_info", rsa_bits).into(),
+        ));
     }
 }
 
 /// Get all entries from algo_info with matching `oid` and `key_type`.
-fn check_card_algo_ecc(
-    algo_info: AlgoInfo,
-    key_type: KeyType,
-    oid: &[u8],
-) -> Vec<EccAttrs> {
+fn check_card_algo_ecc(algo_info: AlgoInfo, key_type: KeyType, oid: &[u8]) -> Vec<EccAttrs> {
     // Find suitable algorithm parameters (from card's list of algorithms).
 
     // Get Algos for this keytype
@@ -544,7 +522,7 @@ fn control_reference_template(key_type: KeyType) -> Result<Tlv, Error> {
         KeyType::Decryption => 0xB8,
         KeyType::Signing => 0xB6,
         KeyType::Authentication => 0xA4,
-        _ => return Err(Error::InternalError(anyhow!("Unexpected KeyType"))),
+        _ => return Err(Error::InternalError("Unexpected KeyType".to_string())),
     };
     Ok(Tlv::new([tag], Value::S(vec![])))
 }
