@@ -1,7 +1,7 @@
 // SPDX-FileCopyrightText: 2021 Heiko Schaefer <heiko@schaefer.name>
 // SPDX-License-Identifier: MIT OR Apache-2.0
 
-//! Access library for
+//! Client library for
 //! [OpenPGP card](https://en.wikipedia.org/wiki/OpenPGP_card)
 //! devices (such as Gnuk, Yubikey, or Java smartcards running an OpenPGP
 //! card application).
@@ -13,8 +13,8 @@
 //! [OpenPGP implementation](https://www.openpgp.org/software/developer/).
 //!
 //! This library can't directly access cards by itself. Instead, users
-//! need to supply an implementation of the [`CardClient`] trait, to
-//! access cards.
+//! need to supply an implementation of the [`CardBackend`]
+//! / [`CardTransaction`] traits, to access cards.
 //!
 //! The companion crate
 //! [openpgp-card-pcsc](https://crates.io/crates/openpgp-card-pcsc)
@@ -27,42 +27,50 @@
 
 pub mod algorithm;
 pub(crate) mod apdu;
-mod card_app;
 pub mod card_do;
 pub mod crypto_data;
 mod errors;
 pub(crate) mod keys;
+mod openpgp;
 mod tlv;
 
-pub use crate::apdu::response::Response;
-pub use crate::card_app::CardApp;
 pub use crate::errors::{Error, SmartcardError, StatusBytes};
+pub use crate::openpgp::{OpenPgp, OpenPgpTransaction};
 
-use anyhow::Result;
 use std::convert::TryInto;
+use std::ops::{Deref, DerefMut};
 
 use crate::apdu::commands;
+use crate::card_do::ApplicationRelatedData;
+use crate::tlv::{tag::Tag, value::Value, Tlv};
 
-/// The CardClient trait defines communication with an OpenPGP card via a
+/// The CardBackend trait defines a connection with an OpenPGP card via a
+/// backend implementation (e.g. via the pcsc backend in the crate
+/// [openpgp-card-pcsc](https://crates.io/crates/openpgp-card-pcsc)),
+/// A CardBackend is only used to get access to a `CardTransaction` object.
+#[blanket::blanket(derive(Box))]
+pub trait CardBackend {
+    fn transaction(&mut self) -> Result<Box<dyn CardTransaction + Send + Sync + '_>, Error>;
+}
+
+/// The CardTransaction trait defines communication with an OpenPGP card via a
 /// backend implementation (e.g. the pcsc backend in the crate
-/// [openpgp-card-pcsc](https://crates.io/crates/openpgp-card-pcsc)).
-pub trait CardClient {
+/// [openpgp-card-pcsc](https://crates.io/crates/openpgp-card-pcsc)),
+/// after opening a transaction from a CardBackend.
+#[blanket::blanket(derive(Box))]
+pub trait CardTransaction {
     /// Transmit the command data in `cmd` to the card.
     ///
     /// `buf_size` is a hint to the backend (the backend may ignore it)
     /// indicating the expected maximum response size.
-    fn transmit(
-        &mut self,
-        cmd: &[u8],
-        buf_size: usize,
-    ) -> Result<Vec<u8>, Error>;
+    fn transmit(&mut self, cmd: &[u8], buf_size: usize) -> Result<Vec<u8>, Error>;
 
-    /// Set the card capabilities in the CardClient.
+    /// Set the card capabilities in the CardTransaction.
     ///
     /// Setting these capabilities is typically part of a bootstrapping
     /// process: the information about the card's capabilities is typically
-    /// requested from the card using the same CardClient instance, before
-    /// the card's capabilities have been initialized.
+    /// requested from the card using the same CardTransaction instance,
+    /// before the card's capabilities have been initialized.
     fn init_card_caps(&mut self, caps: CardCaps);
 
     /// Request the card's capabilities
@@ -71,7 +79,7 @@ pub trait CardClient {
     /// determine if extended length can be used)
     fn card_caps(&self) -> Option<&CardCaps>;
 
-    /// If a CardClient implementation introduces an additional,
+    /// If a CardTransaction implementation introduces an additional,
     /// backend-specific limit for maximum number of bytes per command,
     /// this fn can indicate that limit by returning `Some(max_cmd_len)`.
     fn max_cmd_len(&self) -> Option<usize> {
@@ -85,29 +93,114 @@ pub trait CardClient {
     fn feature_pinpad_modify(&self) -> bool;
 
     /// Verify the PIN `id` via the reader pinpad
-    fn pinpad_verify(&mut self, id: u8) -> Result<Vec<u8>>;
+    fn pinpad_verify(&mut self, pin: PinType) -> Result<Vec<u8>, Error>;
 
     /// Modify the PIN `id` via the reader pinpad
-    fn pinpad_modify(&mut self, id: u8) -> Result<Vec<u8>>;
-}
+    fn pinpad_modify(&mut self, pin: PinType) -> Result<Vec<u8>, Error>;
 
-/// A boxed CardClient (which is Send+Sync).
-pub type CardClientBox = Box<dyn CardClient + Send + Sync>;
-
-impl dyn CardClient {
     /// Select the OpenPGP card application
-    pub fn select(&mut self) -> Result<Response, Error> {
+    fn select(&mut self) -> Result<Vec<u8>, Error> {
         let select_openpgp = commands::select_openpgp();
         apdu::send_command(self, select_openpgp, false)?.try_into()
     }
+
+    /// Get the "application related data" from the card.
+    ///
+    /// (This data should probably be cached in a higher layer. Some parts of
+    /// it are needed regularly, and it does not usually change during
+    /// normal use of a card.)
+    fn application_related_data(&mut self) -> Result<ApplicationRelatedData, Error> {
+        let ad = commands::application_related_data();
+        let resp = apdu::send_command(self, ad, true)?;
+        let value = Value::from(resp.data()?, true)?;
+
+        log::debug!(" ARD value: {:x?}", value);
+
+        Ok(ApplicationRelatedData(Tlv::new(Tag::from([0x6E]), value)))
+    }
+
+    /// Get a CardApp based on a CardTransaction.
+    ///
+    /// It is expected that SELECT has already been performed on the card
+    /// beforehand.
+    ///
+    /// This fn initializes the CardCaps by requesting
+    /// application_related_data from the card, and setting the
+    /// capabilities accordingly.
+    fn initialize(&mut self) -> Result<(), Error> {
+        let ard = self.application_related_data()?;
+
+        // Determine chaining/extended length support from card
+        // metadata and cache this information in the CardTransaction
+        // implementation (as a CardCaps)
+        let mut ext_support = false;
+        let mut chaining_support = false;
+
+        if let Ok(hist) = ard.historical_bytes() {
+            if let Some(cc) = hist.card_capabilities() {
+                chaining_support = cc.command_chaining();
+                ext_support = cc.extended_lc_le();
+            }
+        }
+
+        let ext_cap = ard.extended_capabilities()?;
+
+        // Get max command/response byte sizes from card
+        let (max_cmd_bytes, max_rsp_bytes) =
+            if let Ok(Some(eli)) = ard.extended_length_information() {
+                // In card 3.x, max lengths come from ExtendedLengthInfo
+                (eli.max_command_bytes(), eli.max_response_bytes())
+            } else if let (Some(cmd), Some(rsp)) = (ext_cap.max_cmd_len(), ext_cap.max_resp_len()) {
+                // In card 2.x, max lengths come from ExtendedCapabilities
+                (cmd, rsp)
+            } else {
+                // Fallback: use 255 if we have no information from the card
+                (255, 255)
+            };
+
+        let pw_status = ard.pw_status_bytes()?;
+        let pw1_max = pw_status.pw1_max_len();
+        let pw3_max = pw_status.pw3_max_len();
+
+        let caps = CardCaps {
+            ext_support,
+            chaining_support,
+            max_cmd_bytes,
+            max_rsp_bytes,
+            pw1_max_len: pw1_max,
+            pw3_max_len: pw3_max,
+        };
+
+        log::debug!("init_card_caps to: {:x?}", caps);
+
+        self.init_card_caps(caps);
+
+        Ok(())
+    }
 }
 
-/// Configuration of the capabilities of a card.
+impl<'a> Deref for dyn CardTransaction + Send + Sync + 'a {
+    type Target = dyn CardTransaction + 'a;
+
+    fn deref(&self) -> &Self::Target {
+        self
+    }
+}
+impl<'a> DerefMut for dyn CardTransaction + Send + Sync + 'a {
+    fn deref_mut(&mut self) -> &mut Self::Target {
+        self
+    }
+}
+
+/// Information about the capabilities of a card.
 ///
-/// This configuration is used to determine e.g. if chaining or extended
+/// This configuration can is used to determine e.g. if chaining or extended
 /// length can be used when communicating with the card.
 ///
-/// (This configuration is retrieved from card metadata, specifically from
+/// CardCaps data is used internally and can be used by card backends, it is unlikely to be
+/// useful for users of this library.
+///
+/// (The information is retrieved from card metadata, specifically from
 /// "Card Capabilities", "Extended length information" and "PWStatus")
 #[derive(Clone, Copy, Debug)]
 pub struct CardCaps {
@@ -148,6 +241,23 @@ impl CardCaps {
     }
 }
 
+#[derive(Debug, Clone, Copy, Eq, PartialEq)]
+pub enum PinType {
+    Sign,
+    User,
+    Admin,
+}
+
+impl PinType {
+    pub fn id(&self) -> u8 {
+        match self {
+            PinType::Sign => 0x81,
+            PinType::User => 0x82,
+            PinType::Admin => 0x83,
+        }
+    }
+}
+
 /// Identify a Key slot on an OpenPGP card
 #[derive(Debug, Clone, Copy, Eq, PartialEq)]
 #[non_exhaustive]
